var myProductName = "userlandStorage", myVersion = "0.4.0";

exports.start = start;

exports.initFile = initFile;
exports.getFileId = getFileId;
exports.createFile = createFile;
exports.updateFile = updateFile;
exports.readFile = readFile;
exports.deleteFile = deleteFile;
exports.servePublicFile = servePublicFile;

const fs = require ("fs");
const utils = require ("daveutils"); 
const davesql = require ("davesql");

var config = {
	urlPublicUserFiles: "http://somedomain.com/"
	};

function mergeOptions (userOptions, options) {
	if (userOptions !== undefined) {
		for (x in userOptions) {
			if (userOptions [x] !== undefined) {
				options [x] = userOptions [x];
				}
			}
		}
	}
function encodeValuesForCreating (theStruct) {
	for (var x in theStruct) { //convert booleans
		if (theStruct [x] !== undefined) {
			var val = theStruct [x];
			if (typeof val === "boolean") {
				val = (val) ? 1 : 0;
				}
			theStruct [x] = val;
			}
		}
	const encodedValues = davesql.encodeValues (theStruct);
	return (encodedValues);
	}
function encodeValuesForUpdating (theValues) { 
	var s = "";
	for (var x in theValues) {
		if (s.length > 0) {
			s += ", ";
			}
		s += x + "=" + davesql.encode (theValues [x]);
		}
	return (s);
	}
function getExtraData (fileRec) {
	function getUrlPublic (fileRec) {
		var urlPublic = undefined;
		if (!fileRec.flprivate) { //5/16/24 by DW
			urlPublic = config.urlPublicUserFiles + fileRec.username + "/" + fileRec.relpath;
			}
		return (urlPublic);
		}
	const extraData = {
		urlPublic: getUrlPublic (fileRec)
		}
	return (extraData);
	}

function getFileId (fileRec, callback) {
	var sqltext = "select * from storage where username=" + davesql.encode (fileRec.username) + " and relpath=" + davesql.encode (fileRec.relpath) + ";";
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (err);
			}
		else {
			if (result.length == 0) {
				const message = "Can't find the file " + fileRec.relpath + " for the user " + fileRec.username + ".";
				callback ({message});
				}
			else {
				const theFileRec = result [0];
				callback (undefined, theFileRec.id);
				}
			}
		});
	}
function initFile (fileRec, callback) {
	getFileId (fileRec, function (err, id) {
		if (err) {
			createFile (fileRec, function (err, theNewFile) {
				if (err) {
					callback (err);
					}
				else {
					callback (undefined, theNewFile.id);
					}
				});
			}
		else {
			callback (undefined, id);
			}
		});
	}
function readFile (idFile, callback) {
	var sqltext = "select * from storage where id = " + davesql.encode (idFile) + ";";
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (err);
			}
		else {
			if (result.length == 0) {
				const message = "Can't find the file " + relpath + " for the user " + username + ".";
				callback ({message});
				}
			else {
				const theFileRec = result [0];
				callback (undefined, theFileRec);
				}
			}
		});
	}
function createFile (fileRecParam, callback) { 
	var fileRec = {
		username: undefined, 
		relpath: undefined, 
		type: "text/plain",
		filecontents: "",
		flprivate: true,
		metadata: "{}"
		};
	mergeOptions (fileRecParam, fileRec);
	
	const sqltext = "insert into storage " + encodeValuesForCreating (fileRec);
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (err);
			}
		else {
			fileRec.id = result.insertId;
			callback (undefined, fileRec, getExtraData (fileRec));
			}
		});
	}
function updateFile (idfile, fileRec, callback) {
	var newFileRec = new Object (); //7/18/24 by DW
	for (var x in fileRec) {
		if (utils.stringLower (x) != "whenupdated") {
			newFileRec [x] = fileRec [x];
			}
		}
	const sqltext = "update storage set " + encodeValuesForUpdating (newFileRec) + ", ctUpdates = ctUpdates + 1 where id = " + davesql.encode (idfile) + ";";
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (err);
			}
		else {
			if (result.affectedRows == 0) {
				const message = "Can't update the file because there is no record with id == " + idfile;
				callback ({message});
				}
			else {
				fileRec.id = idfile;
				callback (undefined, fileRec, getExtraData  (fileRec));
				}
			}
		});
	}
function deleteFile (id, callback) {
	const sqltext = "delete from storage where id = " + davesql.encode (id) + ";";
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (err);
			}
		else {
			if (result.length == 0) {
				const message = "Can't delete the file because there is no file with id == " + id;
				callback ({message});
				}
			else {
				callback (undefined, true);
				}
			}
		});
	}

function servePublicFile (relpath, callback) { //4/30/24 by DW
	if (utils.beginsWith (relpath, "/")) {
		relpath = utils.stringDelete (relpath, 1, 1);
		}
	const parts = relpath.split ("/");
	if (parts.length < 2) {
		return (false);
		}
	const username = parts [0];
	if (username.length == 0) {
		return (false);
		}
	
	relpath = utils.stringDelete (relpath, 1, username.length + 1);
	if (relpath.length == 0) {
		return (false);
		}
	
	const sqltext = "select * from  storage where username = " + davesql.encode (username) + " and relpath = " + davesql.encode (relpath) + ";";
	console.log ("\nservePublicFile: relpath == " + relpath + ", sqltext == " + sqltext + "\n");
	
	davesql.runSqltext (sqltext, function (err, result) {
		if (err) {
			callback (500, "text/plain", err.message);
			}
		else {
			if (result.length == 0) {
				callback (404, "text/plain", "Not found");
				}
			else {
				const fileRec = result [0];
				if (utils.getBoolean (fileRec.flprivate)) {
					callback (403, "text/plain", "Forbidden");
					}
				else {
					callback (200, fileRec.type, fileRec.filecontents);
					}
				}
			}
		});
	
	return (true); //the request was for us
	}

function start (userOptions, callback) {
	mergeOptions (userOptions, config);
	callback ();
	}
